/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity;

import java.util.Set;
import java.util.TreeSet;

import net.sf.parser4j.parser.entity.data.TerminalCharRange;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.DebugUtil;
import net.sf.parser4j.parser.service.ParserException;

/**
 * parse result
 * 
 * @author luc peuvrier
 * 
 */
public class ParseResult {

	/** true if error during parsing */
	private final boolean inError;// NOPMD

	/** true if syntax parsing error */
	private boolean parseError;

	/** true if reduce error while parsing */
	private boolean reduceError;

	/** file name where is syntax error */
	private String fileName; // NOPMD

	/** line number in input text file for syntax error */
	private int lineNumber = -1;// NOPMD

	/** column number in input text file for syntax error */
	private int columnNumber = -1;// NOPMD

	/** expected terminal not found in input text file for syntax error */
	private final Set<TerminalCharRange> expectedTerminal = // NOPMD
	/**/new TreeSet<TerminalCharRange>();

	/** expected non terminal not found in input text file for syntax error */
	private final Set<String> expectedNonTerminal = // NOPMD
	/**/new TreeSet<String>();

	/** the reduce error cause for reduce error */
	private ParserException reduceException;// NOPMD

	/** file name where is reduce error */
	private String reduceErrorFileName; // NOPMD

	/** line number in input text file for reduce error */
	private int reduceErrorLineNumber;// NOPMD

	/** column number in input text file for reduce error */
	private int reduceErrorColumnNumber;// NOPMD

	/** the resulting parse tree root node if no syntax error */
	private IParseNode resultParseNode;

	/**
	 * set error state at creation
	 * 
	 * @param inError
	 *            true if error during parsing
	 */
	public ParseResult(final boolean inError) {
		this.inError = inError;
	}

	/**
	 * set the resulting parse tree root node
	 * 
	 * @param resultParseNode
	 */
	public void setResultParseNode(final IParseNode resultParseNode) {
		this.resultParseNode = resultParseNode;
	}

	/**
	 * set line number and column number in input text file for syntax error
	 * 
	 * @param lineNumber
	 *            line in file where is syntax error
	 * @param columnNumber
	 *            column in file where is syntax error
	 * @param fileName
	 *            file name where is syntax error
	 */
	public void setInTextPosition(final int lineNumber, final int columnNumber,
			final String fileName) {
		this.fileName = fileName;
		this.lineNumber = lineNumber;
		this.columnNumber = columnNumber;
	}

	/**
	 * set reduce error state
	 * 
	 * @param reduceError
	 *            true if reduce error while parsing
	 */
	public void setReduceError(final boolean reduceError) {
		this.reduceError = reduceError;
	}

	/**
	 * set syntax error state
	 * 
	 * @param parseError
	 *            true if syntax parsing error
	 */
	public void setParseError(final boolean parseError) {
		this.parseError = parseError;
	}

	/**
	 * set the reduce error informations
	 * 
	 * @param reduceException
	 *            the reduce error cause for reduce error
	 * @param reducErrorLineNumber
	 *            line number in input text file for reduce error
	 * @param reduceErrorColumnNumber
	 *            column number in input text file for reduce error
	 * @param reduceErrorFileName
	 *            file name where is reduce error
	 */
	public void setReduceErrorInfo(final ParserException reduceException,
			final int reducErrorLineNumber, final int reduceErrorColumnNumber,
			final String reduceErrorFileName) {
		this.reduceErrorFileName = reduceErrorFileName;
		this.reduceException = reduceException;
		this.reduceErrorLineNumber = reducErrorLineNumber;
		this.reduceErrorColumnNumber = reduceErrorColumnNumber;
	}

	/**
	 * 
	 * @return true if error during parsing
	 */
	public boolean isInError() {
		return inError;
	}

	/**
	 * 
	 * @return the resulting parse tree root node if no syntax error
	 */
	public IParseNode getResultParseNode() {
		return resultParseNode;
	}

	/**
	 * 
	 * @return true if syntax parsing error
	 */
	public boolean isParseError() {
		return parseError;
	}

	/**
	 * 
	 * @return true if reduce error while parsing
	 */
	public boolean isReduceError() {
		return reduceError;
	}

	/**
	 * 
	 * @return file name where is syntax error
	 */
	public String getFileName() {
		return fileName;
	}

	/**
	 * 
	 * @return column number in input text file for syntax error
	 */
	public int getColumnNumber() {
		return columnNumber;
	}

	/**
	 * 
	 * @return line number in input text file for syntax error
	 */
	public int getLineNumber() {
		return lineNumber;
	}

	/**
	 * 
	 * @return expected non terminal not found in input text file for syntax
	 *         error
	 */
	public Set<String> getExpectedNonTerminal() {
		return expectedNonTerminal;
	}

	/**
	 * 
	 * @return expected terminal not found in input text file for syntax error
	 */
	public Set<TerminalCharRange> getExpectedTerminal() {
		return expectedTerminal;
	}

	/**
	 * 
	 * @return the reduce error cause for reduce error
	 */
	public ParserException getReduceException() {
		return reduceException;
	}

	/**
	 * 
	 * @return file name where is reduce error
	 */
	public String getReduceErrorFileName() {
		return reduceErrorFileName;
	}

	/**
	 * 
	 * @return line number in input text file for reduce error
	 */
	public int getReduceErrorLineNumber() {
		return reduceErrorLineNumber;
	}

	/**
	 * 
	 * @return column number in input text file for reduce error
	 */
	public int getReduceErrorColumnNumber() {
		return reduceErrorColumnNumber;
	}

	@Override
	public String toString() {
		final StringBuilder stringBuilder = new StringBuilder();
		if (inError) {
			stringBuilder.append("has parsing error\n");
		} else {
			stringBuilder.append("does not have parsing error\n");
		}
		if (parseError) {
			stringBuilder.append("parse failed: file ");
			stringBuilder.append(fileName);
			stringBuilder.append(", line ");
			stringBuilder.append(lineNumber);
			stringBuilder.append(" column ");
			stringBuilder.append(columnNumber);
			stringBuilder.append("\nexpected terminal:\n");
			for (TerminalCharRange terminalCharRange : expectedTerminal) {
				stringBuilder.append('\t');
				stringBuilder.append(terminalCharRange.toString());
				stringBuilder.append('\n');
			}
			stringBuilder.append("expected non terminal:\n");
			for (String nonTerminal : expectedNonTerminal) {
				stringBuilder.append('\t');
				stringBuilder.append(nonTerminal);
				stringBuilder.append('\n');
			}
		} else {
			stringBuilder.append("parse succeed\n");
		}
		if (reduceError) {
			final DebugUtil debugUtil = DebugUtil.getInstance();
			stringBuilder.append("reduce error: file ");
			stringBuilder.append(reduceErrorFileName);
			stringBuilder.append(", line ");
			stringBuilder.append(reduceErrorLineNumber);
			stringBuilder.append(", column ");
			stringBuilder.append(reduceErrorColumnNumber);
			stringBuilder.append('\n');
			stringBuilder.append(debugUtil.stackTrace(reduceException));
		} else {
			stringBuilder.append("no reduce error\n");
		}
		return stringBuilder.toString();
	}
}
