/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.service.parsenode;

import java.util.Deque;
import java.util.IdentityHashMap;
import java.util.Map;

import net.sf.parser4j.parser.entity.EnumNodeType;
import net.sf.parser4j.parser.entity.data.NonTerminal;
import net.sf.parser4j.parser.entity.parsenode.CharacterParseNode;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.entity.parsenode.StringParseNode;
import net.sf.parser4j.parser.entity.parsestate.IParseState;
import net.sf.parser4j.parser.entity.parsestate.ParseStack;
import net.sf.parser4j.parser.entity.parsestate.ParseStackElement;
import net.sf.parser4j.parser.service.Parser;

import org.apache.log4j.Logger;

/**
 * parse node, parse tree, to string utilities
 * 
 * @author luc peuvrier
 * 
 */
public final class ParsingToStringUtil implements IParseNodeVisitor {

	private static final Logger _log = Logger
			.getLogger(ParsingToStringUtil.class);

	private static final ParsingToStringUtil INSTANCE =
	/**/new ParsingToStringUtil();

	private transient StringBuilder stringBuilder;

	private transient int numberOfTab;// NOPMD

	/** non terminal by non terminal identifier */
	private Map<Integer, NonTerminal> nonTerminalByIdentifierMap;// NOPMD

	private final transient IdentityHashMap<IParseNode, Boolean> visitedSet = // NOPMD
	/**/new IdentityHashMap<IParseNode, Boolean>();

	public static ParsingToStringUtil getInstance() {
		return INSTANCE;
	}

	private ParsingToStringUtil() {
		super();
	}

	public void setNonTerminalByIdentifierMap(
			final Map<Integer, NonTerminal> nonTerminalByIdentifierMap) {
		this.nonTerminalByIdentifierMap = nonTerminalByIdentifierMap;
	}

	/**
	 * parse state to string
	 * 
	 * @param parseState
	 * @param tokenRecognition
	 * @param addParseTree
	 *            true if add parse tree for each node of parse nodes array
	 * @return
	 */
	public String parseStateToString(final IParseState parseState,
			final boolean[] tokenRecognition, final boolean addParseTree) {
		final StringBuilder builder = new StringBuilder();
		if (parseState.isEmpty()) {
			builder.append("empty parse state\n");
		} else {
			for (ParseStack parseStack : parseState) {
				builder.append(parseStackToString(parseStack, tokenRecognition,
						addParseTree));
				builder.append('\n');
			}
		}
		return builder.toString();
	}

	/**
	 * parse stack to string
	 * 
	 * @param parseStack
	 * @param tokenRecognition
	 * @param addParseTree
	 *            true if add parse tree for each node of parse nodes array
	 * @return
	 */
	public String parseStackToString(final ParseStack parseStack,
			final boolean[] tokenRecognition, final boolean addParseTree) {
		final StringBuilder builder = new StringBuilder();
		final Deque<ParseStackElement> que = parseStack
				.getParseStateNodeStack();
		builder.append("stack begin\n");
		if (parseStack.isMatchAll()) {
			builder.append("match all\n");
		}
		int stateNumber;
		int index = 0;
		for (ParseStackElement parseStackElement : que) {
			builder.append('#');
			builder.append(index);
			builder.append(" : ");
			stateNumber = parseStackElement.getStateNumber();
			builder.append(stateNumber);
			if (tokenRecognition[stateNumber]) {
				builder.append("(token)");
			}
			builder.append(" , ");
			final IParseNode parseNode = parseStackElement.getParseNode();
			parseNodeToString(builder, parseNode);
			builder.append('\n');
			if (addParseTree) {
				builder.append(parseTreeToString(parseNode));
				builder.append('\n');
			}
			index++;
		}
		builder.append('#');
		builder.append(index);
		builder.append(" : ");
		stateNumber = parseStack.getStateNumber();
		builder.append(stateNumber);
		if (stateNumber != Parser.ENDING_STATE_NUMBER
				&& tokenRecognition[stateNumber]) {
			builder.append('*');
		}
		builder.append('\n');
		builder.append("stack end\n");
		return builder.toString();
	}

	/**
	 * parse nodes to string
	 * 
	 * @param parseNodes
	 * @param addParseTree
	 *            true if add parse tree for each node of parse nodes array
	 * @return
	 */
	public String parseNodesToString(final IParseNode[] parseNodes,
			final boolean addParseTree) {
		final StringBuilder builder = new StringBuilder();
		int index = 0;
		for (IParseNode parseNode : parseNodes) {
			builder.append('#');
			builder.append(index++);
			builder.append(": ");
			parseNodeToString(builder, parseNode);
			builder.append('\n');
			if (addParseTree) {
				builder.append(parseTreeToString(parseNode));
			}
		}
		return builder.toString();
	}

	/**
	 * parse node to string
	 * 
	 * @param parseNode
	 * @return
	 */
	public String parseNodeToString(final IParseNode parseNode) {
		final StringBuilder builder = new StringBuilder();
		parseNodeToString(builder, parseNode);
		return builder.toString();
	}

	/**
	 * parse tree to string
	 * 
	 * @param rootParseNode
	 * @return
	 */
	public String parseTreeToString(final IParseNode rootParseNode) {
		visitedSet.clear();
		stringBuilder = new StringBuilder();
		numberOfTab = 0;
		try {
			rootParseNode.accept(this);
		} catch (Exception exception) {
			_log.error("visiting", exception);
		}
		final String toString = stringBuilder.toString();
		visitedSet.clear();
		return toString;
	}

	public boolean beginVisit(final IParseNode parseNode) {
		final boolean visited = visitedSet.containsKey(parseNode);
		addTab();
		stringBuilder.append("begin ");
		parseNodeToString(stringBuilder, parseNode);
		stringBuilder.append('\n');
		final boolean visitSon = true ^ visited;// !visited
		if (visitSon) {
			visitedSet.put(parseNode, Boolean.TRUE);
			// visit son
			numberOfTab++;
		}
		return visitSon;
	}

	@Override
	public boolean visitFirstAlternativeOnly() {
		return false;
	}

	@Override
	public void beginAlternativeVisit(final IParseNode node,
			final int alternativeNumber) {
		if (node.getNumberOfAlternative() > 1) {
			addTab();
			stringBuilder.append("begin alternative #");
			stringBuilder.append(alternativeNumber);
			stringBuilder.append(' ');
			stringBuilder.append(grammarSymbolIdentifierToString(node
					.getGrammarSymbolIdentifier()));
			stringBuilder.append('\n');
			numberOfTab++;
		}
	}

	@Override
	public void endAlternativeVisit(final IParseNode node,
			final int alternativeNumber) {
		if (node.getNumberOfAlternative() > 1) {
			numberOfTab--;
			addTab();
			stringBuilder.append("end alternative #");
			stringBuilder.append(alternativeNumber);
			stringBuilder.append('\n');
		}
	}

	public void endVisit(final IParseNode node) {
		numberOfTab--;
		addTab();
		stringBuilder.append("end ");
		stringBuilder.append(grammarSymbolIdentifierToString(node
				.getGrammarSymbolIdentifier()));
		stringBuilder.append('\n');
	}

	private void addTab() {
		for (int count = 0; count < numberOfTab; count++) {
			stringBuilder.append(' ');
		}
	}

	/**
	 * update stringBuiler with parse node to string
	 * 
	 * @param stringBuilder
	 * @param parseNode
	 */
	private void parseNodeToString(final StringBuilder stringBuilder,
			final IParseNode parseNode) {
		final int grammarSymbolIdentifier = parseNode
				.getGrammarSymbolIdentifier();
		final EnumNodeType parseNodeType = parseNode.getParseNodeType();
		stringBuilder.append(parseNodeType);
		stringBuilder.append(' ');
		stringBuilder
				.append(grammarSymbolIdentifierToString(grammarSymbolIdentifier));
		stringBuilder.append('(');
		stringBuilder.append(grammarSymbolIdentifier);
		stringBuilder.append(')');
		if (EnumNodeType.CHARACTER_TERMINAL.equals(parseNodeType)) {
			final int terminalIdentifier = ((CharacterParseNode) parseNode)
					.getTerminalValue();
			stringBuilder.append(' ');
			stringBuilder
					.append(terminalIdentifierToString(terminalIdentifier));
		} else if (EnumNodeType.STRING_TERMINAL.equals(parseNodeType)) {
			final String value = ((StringParseNode) parseNode).getStringValue();
			stringBuilder.append(' ');
			stringBuilder.append(value);
		}
	}

	/**
	 * grammar symbol identifier to string
	 * 
	 * @param grammarSymbolIdentifier
	 * @return
	 */
	public String grammarSymbolIdentifierToString(
			final int grammarSymbolIdentifier) {
		final String toString;
		if (grammarSymbolIdentifier < 0x10000) {
			toString = terminalIdentifierToString(grammarSymbolIdentifier);
		} else {
			final NonTerminal nonTerminal = nonTerminalByIdentifierMap
					.get(grammarSymbolIdentifier);
			if (nonTerminal.isToken()) {
				toString = nonTerminal.getName() + " (token)";
			} else {
				toString = nonTerminal.getName();
			}
		}
		return toString;
	}

	/**
	 * terminal identifier to string : the character for ascii, or the utf code
	 * point \\uXXXX
	 * 
	 * @param terminalIdentifier
	 * @return
	 */
	public String terminalIdentifierToString(final int terminalIdentifier) {
		final String toString;
		if (terminalIdentifier < ' ' || terminalIdentifier >= 0x7f) {
			toString = String.format("\\u%04X",
					new Object[] { terminalIdentifier });
		} else {
			toString = "'" + ((char) terminalIdentifier) + "'";
		}
		return toString;
	}
}
